#!/usr/local/bin/perl
#
# Copyright (c) 2005 Erik Mugele.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Designed and written by Erik Mugele, 2004-2005
# http://www.teuton.org/~erik
# Version 1.0
#
# CGI to remove a user from a Subverion repository passwd file.
# Administrative users must be listed in a file called "admins"
# in the same directory as the "passwd" file.  One admin user
# per line.
# Administrator users must exist in the Subversion repository
# passwd file.
#

use CGI;

$eol="\n";
$dirdelim="/";
$repodir="/usr/local/svn".$dirdelim."repos";
$pathsuffix="conf".$dirdelim;

sub printHTMLhead {
    print $query->header;
    print <<EOF;
    <html>
    <!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">
    <head>
    <title>Subversion Remove User</title>
    </head>
    <body>
    <center><h2><font face='arial'>Subversion Remove User</font></h2></center>
EOF
}

sub printHTMLfoot {
    print <<EOF;
    </body>
    </html>
EOF
}

sub printform {
    opendir(dh,$repodir) or die "Directory $repodir does not exist.\n";
    foreach $i (sort readdir(dh)) {
        push(@repolist,$i);
    }
    closedir(dh);
    
    print <<EOF;
    <font face='arial'>Notes:</font>
    <ul>
    <li><font face='arial'>Passwords should not contain whitespace.</font></li>
    <li><font face='arial'>Passwords are stored on the server in plain text and are readable by system administrators.</font></li>
    </ul>
    <hr>
    <form method='post' action='$thisscript'>
    <h3><font face='arial'>Authentication</font></h3>
    <font face='arial'>Enter your username and password.</font><br><br>
    <table>
    <tr>
    <td align='right'><font face='arial'>Repository: </font></td>
    <td>
    <select name='repo'>
EOF
    foreach $i (@repolist) {
        if ($i !~ m/^\./) {
            print "<option value='$i'>$i</option>\n";
        }
    }
    print <<EOF;
    </select>
    </td>
    </tr>
    <tr>
    <td align='right'><font face='arial'>Username: </font></td>
    <td><input type='text' name='authlogin'></td>
    </tr>
    <tr>
    <td align='right'><font face='arial'>Password: </font</td>
    <td><input type='password' name='authpasswd'></td>
    </tr>
    </table>
    <hr>
    <h3><font face='arial'>Remove User</font></h3>
    <font face='arial'>Enter user to be removed.</font><br><br>
    <table>
    <tr>
    <td align='right'><font face='arial'>Username: </font</td>
    <td><input type='text' name='login'></td>
    </tr>
    </table>
    <input type='submit' name='submit' value='Remove User'>
EOF
}

$thisscript=(split($dirdelim,$0))[-1];
$query = new CGI;
if ($query->param("submit")) {
    if (($query->param("repo")) &&
        ($query->param("authlogin")) &&
        ($query->param("authpasswd")) &&
        ($query->param("login"))) {
        $errormessage="";
        $repo=$query->param("repo");
        $authuser=$query->param("authlogin");
        $authpass=$query->param("authpasswd");
        $user=$query->param("login");
        $passwd_file = $repodir.$dirdelim.$repo.$dirdelim.$pathsuffix."passwd";
        $passwd_backup = "$passwd_file~";
        $admins_file = $repodir.$dirdelim.$repo.$dirdelim.$pathsuffix."admins";
        if (not stat $admins_file) {
            $errormessage = "Invalid repository and/or admins file. Someone needs to check in to this.";
        }
        if (not stat $passwd_file) {
            $errormessage = "Invalid repository and/or password file.  Someone needs to check in to this.";
        }
        if ($errormessage eq "") { 
            open (fd, $passwd_file) or die "Can't open $passwd_file.\n";
            while (<fd>) {
                push(@passwd_list,$_);
                next if (/^#/ || /^$/ || /^\[/ || /^\s$/);
                ($tmp_user,$tmp_pass)=split(/=/,$_);
                $tmp_user =~ s/\s//g;
                $tmp_pass =~ s/\s//g;
                $passwd_hash{$tmp_user}=$tmp_pass;
            }
            close(fd) or die "Can't close $passwd_file~.\n";
            open (fd, $admins_file) or die "Can't open $admins_file.\n";
            while (<fd>) {
                next if (/^#/ || /^$/ || /^\s$/);
                chomp($_);
                push (@admins,$_);
            }
            close(fd) or die "Can't close $admins_file.\n";

            if ((not exists $passwd_hash{$authuser}) || ($passwd_hash{$authuser} ne $authpass)) {
                $errormessage = "Invalid authentication username and/or password.";
            } else {
                if (not grep(/^$authuser$/,@admins)) {
                    $errormessage = "You are not an admin for this repository.";
                }
            }
            if ($errormessage eq "") {
                if (not exists $passwd_hash{$user}) {
                    $errormessage = "Username does not exist in the repository.";
                } else {
                    # Backup up the passwd file
                    open (fd, ">$passwd_backup") or die "Can't open $passwd_backup.\n";
                    foreach $i (@passwd_list) {
                        print fd $i;
                    }
                        close (fd) or die "Can't close $passwd_backup.\n";
                    open (fd, ">$passwd_file") or die "Can't open $passwd_file for writing.\n";
                    foreach $i (@passwd_list) {
                        if ($i !~ m/^$user/) {
                            print fd $i;
                        }
                    }
                    close (fd) or die "Can't close $passwd_file.\n";
                }
            }
        }
    } else {
        $errormessage = "Invalid parameters or empty fields.";
    }
    if ($errormessage ne "") {
        printHTMLhead;
        print "<font face='arial' color='red'>$errormessage</font><br><br>$eol";
        printform;
        printHTMLfoot;
    } else {
        printHTMLhead;
        print "<font face='arial' color='green'>User <b>$user</b> has been successfully removed from <b>$repo</b> repository.</font><br><br>$eol";
        printform;
        printHTMLfoot;
    }
} else {
    printHTMLhead;
    printform;
    printHTMLfoot;
}
 
